@@ -50,11 +50,6 @@ static bool quickmatch(prte_node_t *nd, char *name)
     if (0 == strcmp(nd->name, name)) {
         return true;
     }
-    if (0 == strcmp(nd->name, prte_process_info.nodename) &&
-        (0 == strcmp(name, "localhost") ||
-         0 == strcmp(name, "127.0.0.1"))) {
-        return true;
-    }
     if (NULL != nd->aliases) {
         for (n=0; NULL != nd->aliases[n]; n++) {
             if (0 == strcmp(nd->aliases[n], name)) {
@@ -412,7 +407,7 @@ int prte_util_add_dash_host_nodes(pmix_list_t *nodes, char *hosts, bool allocati
         }
     }
 
-    if (prte_managed_allocation) {
+    if (prte_managed_allocation && !allocating) {
         prte_node_t *node_from_pool = NULL;
         PMIX_LIST_FOREACH(node, nodes, prte_node_t) {
             needcheck = true;
