@@ -9,7 +9,8 @@
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2018 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -23,18 +24,22 @@
 #include <string.h>
 
 #include "opal/class/opal_pointer_array.h"
+#include "opal/util/printf.h"
+#include "opal/util/string_copy.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/attribute/attribute.h"
 
 static void __ompi_datatype_allocate( ompi_datatype_t* datatype )
 {
     datatype->args               = NULL;
-    datatype->d_f_to_c_index     = opal_pointer_array_add(&ompi_datatype_f_to_c_table, datatype);
-    /* Later generated datatypes will have their id according to the Fortran ID, as ALL types are registered */
-    datatype->id                 = datatype->d_f_to_c_index;
+    /* Do not add the newly created datatypes to the f2c translation table. We will add them only
+     * if necessary, basically upon the first call the MPI_Datatype_f2c.
+     */
+    datatype->d_f_to_c_index     = -1;
+    datatype->id                 = -1;
     datatype->d_keyhash          = NULL;
     datatype->name[0]            = '\0';
-    datatype->packed_description = NULL;
+    datatype->packed_description = 0;
     datatype->pml_data           = 0;
 }
 
@@ -44,12 +49,13 @@ static void __ompi_datatype_release(ompi_datatype_t * datatype)
         ompi_datatype_release_args( datatype );
         datatype->args = NULL;
     }
-    if( NULL != datatype->packed_description ) {
-        free( datatype->packed_description );
-        datatype->packed_description = NULL;
-    }
-    if( NULL != opal_pointer_array_get_item(&ompi_datatype_f_to_c_table, datatype->d_f_to_c_index) ){
+
+    free ((void *) datatype->packed_description );
+    datatype->packed_description = 0;
+
+    if( datatype->d_f_to_c_index >= 0 ) {
         opal_pointer_array_set_item( &ompi_datatype_f_to_c_table, datatype->d_f_to_c_index, NULL );
+        datatype->d_f_to_c_index = -1;
     }
     /* any pending attributes ? */
     if (NULL != datatype->d_keyhash) {
@@ -105,8 +111,12 @@ ompi_datatype_duplicate( const ompi_datatype_t* oldType, ompi_datatype_t** newTy
        the top level (specifically, MPI_TYPE_DUP). */
     new_ompi_datatype->d_keyhash = NULL;
     new_ompi_datatype->args = NULL;
-    snprintf (new_ompi_datatype->name, MPI_MAX_OBJECT_NAME, "Dup %s",
-              oldType->name);
+
+    char *new_name;
+    opal_asprintf(&new_name, "Dup %s", oldType->name);
+    opal_string_copy(new_ompi_datatype->name, new_name, MPI_MAX_OBJECT_NAME);
+    new_ompi_datatype->name[MPI_MAX_OBJECT_NAME - 1] = '\0';
+    free(new_name);
 
     return OMPI_SUCCESS;
 }
