@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2015 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -14,10 +14,11 @@
  * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2013-2016 Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation.  All rights reserved.
  * Copyright (c) 2017      FUJITSU LIMITED.  All rights reserved.
+ * Copyright (c) 2019      Mellanox Technologies. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -33,6 +34,7 @@
 #include "ompi/mca/coll/base/base.h"
 #include "ompi/mca/mca.h"
 #include "ompi/mca/coll/coll.h"
+#include "ompi/info/info.h"
 #include "ompi/request/request.h"
 
 /* need to include our own topo prototypes so we can malloc data on the comm correctly */
@@ -135,6 +137,29 @@ typedef enum COLLTYPE {
 #define INEIGHBOR_ALLTOALLV_ARGS  NEIGHBOR_ALLTOALLV_BASE_ARGS,  ompi_request_t **request, mca_coll_base_module_t *module
 #define INEIGHBOR_ALLTOALLW_ARGS  NEIGHBOR_ALLTOALLW_BASE_ARGS,  ompi_request_t **request, mca_coll_base_module_t *module
 
+#define ALLGATHER_INIT_ARGS           ALLGATHER_BASE_ARGS,           ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define ALLGATHERV_INIT_ARGS          ALLGATHERV_BASE_ARGS,          ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define ALLREDUCE_INIT_ARGS           ALLREDUCE_BASE_ARGS,           ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define ALLTOALL_INIT_ARGS            ALLTOALL_BASE_ARGS,            ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define ALLTOALLV_INIT_ARGS           ALLTOALLV_BASE_ARGS,           ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define ALLTOALLW_INIT_ARGS           ALLTOALLW_BASE_ARGS,           ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define BARRIER_INIT_ARGS             BARRIER_BASE_ARGS,             ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define BCAST_INIT_ARGS               BCAST_BASE_ARGS,               ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define EXSCAN_INIT_ARGS              EXSCAN_BASE_ARGS,              ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define GATHER_INIT_ARGS              GATHER_BASE_ARGS,              ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define GATHERV_INIT_ARGS             GATHERV_BASE_ARGS,             ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define REDUCE_INIT_ARGS              REDUCE_BASE_ARGS,              ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define REDUCESCATTER_INIT_ARGS       REDUCESCATTER_BASE_ARGS,       ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define REDUCESCATTERBLOCK_INIT_ARGS  REDUCESCATTERBLOCK_BASE_ARGS,  ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define SCAN_INIT_ARGS                SCAN_BASE_ARGS,                ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define SCATTER_INIT_ARGS             SCATTER_BASE_ARGS,             ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define SCATTERV_INIT_ARGS            SCATTERV_BASE_ARGS,            ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define NEIGHBOR_ALLGATHER_INIT_ARGS  NEIGHBOR_ALLGATHER_BASE_ARGS,  ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define NEIGHBOR_ALLGATHERV_INIT_ARGS NEIGHBOR_ALLGATHERV_BASE_ARGS, ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define NEIGHBOR_ALLTOALL_INIT_ARGS   NEIGHBOR_ALLTOALL_BASE_ARGS,   ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define NEIGHBOR_ALLTOALLV_INIT_ARGS  NEIGHBOR_ALLTOALLV_BASE_ARGS,  ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+#define NEIGHBOR_ALLTOALLW_INIT_ARGS  NEIGHBOR_ALLTOALLW_BASE_ARGS,  ompi_info_t *info, ompi_request_t **request, mca_coll_base_module_t *module
+
 #define ALLGATHER_BASE_ARG_NAMES           sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm
 #define ALLGATHERV_BASE_ARG_NAMES          sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm
 #define ALLREDUCE_BASE_ARG_NAMES           sendbuf, recvbuf, count, datatype, op, comm
@@ -164,6 +189,7 @@ BEGIN_C_DECLS
 /* All Gather */
 int ompi_coll_base_allgather_intra_bruck(ALLGATHER_ARGS);
 int ompi_coll_base_allgather_intra_recursivedoubling(ALLGATHER_ARGS);
+int ompi_coll_base_allgather_intra_sparbit(ALLGATHER_ARGS);
 int ompi_coll_base_allgather_intra_ring(ALLGATHER_ARGS);
 int ompi_coll_base_allgather_intra_neighborexchange(ALLGATHER_ARGS);
 int ompi_coll_base_allgather_intra_basic_linear(ALLGATHER_ARGS);
@@ -171,6 +197,7 @@ int ompi_coll_base_allgather_intra_two_procs(ALLGATHER_ARGS);
 
 /* All GatherV */
 int ompi_coll_base_allgatherv_intra_bruck(ALLGATHERV_ARGS);
+int ompi_coll_base_allgatherv_intra_sparbit(ALLGATHERV_ARGS);
 int ompi_coll_base_allgatherv_intra_ring(ALLGATHERV_ARGS);
 int ompi_coll_base_allgatherv_intra_neighborexchange(ALLGATHERV_ARGS);
 int ompi_coll_base_allgatherv_intra_basic_default(ALLGATHERV_ARGS);
@@ -182,6 +209,7 @@ int ompi_coll_base_allreduce_intra_recursivedoubling(ALLREDUCE_ARGS);
 int ompi_coll_base_allreduce_intra_ring(ALLREDUCE_ARGS);
 int ompi_coll_base_allreduce_intra_ring_segmented(ALLREDUCE_ARGS, uint32_t segsize);
 int ompi_coll_base_allreduce_intra_basic_linear(ALLREDUCE_ARGS);
+int ompi_coll_base_allreduce_intra_redscat_allgather(ALLREDUCE_ARGS);
 
 /* AlltoAll */
 int ompi_coll_base_alltoall_intra_pairwise(ALLTOALL_ARGS);
@@ -220,8 +248,14 @@ int ompi_coll_base_bcast_intra_pipeline(BCAST_ARGS, uint32_t segsize);
 int ompi_coll_base_bcast_intra_binomial(BCAST_ARGS, uint32_t segsize);
 int ompi_coll_base_bcast_intra_bintree(BCAST_ARGS, uint32_t segsize);
 int ompi_coll_base_bcast_intra_split_bintree(BCAST_ARGS, uint32_t segsize);
+int ompi_coll_base_bcast_intra_knomial(BCAST_ARGS, uint32_t segsize, int radix);
+int ompi_coll_base_bcast_intra_scatter_allgather(BCAST_ARGS, uint32_t segsize);
+int ompi_coll_base_bcast_intra_scatter_allgather_ring(BCAST_ARGS, uint32_t segsize);
 
 /* Exscan */
+int ompi_coll_base_exscan_intra_recursivedoubling(EXSCAN_ARGS);
+int ompi_coll_base_exscan_intra_linear(EXSCAN_ARGS);
+int ompi_coll_base_exscan_intra_recursivedoubling(EXSCAN_ARGS);
 
 /* Gather */
 int ompi_coll_base_gather_intra_basic_linear(GATHER_ARGS);
@@ -238,17 +272,29 @@ int ompi_coll_base_reduce_intra_pipeline(REDUCE_ARGS, uint32_t segsize, int max_
 int ompi_coll_base_reduce_intra_binary(REDUCE_ARGS, uint32_t segsize, int max_outstanding_reqs );
 int ompi_coll_base_reduce_intra_binomial(REDUCE_ARGS, uint32_t segsize, int max_outstanding_reqs );
 int ompi_coll_base_reduce_intra_in_order_binary(REDUCE_ARGS, uint32_t segsize, int max_outstanding_reqs );
+int ompi_coll_base_reduce_intra_redscat_gather(REDUCE_ARGS);
 
 /* Reduce_scatter */
 int ompi_coll_base_reduce_scatter_intra_nonoverlapping(REDUCESCATTER_ARGS);
 int ompi_coll_base_reduce_scatter_intra_basic_recursivehalving(REDUCESCATTER_ARGS);
 int ompi_coll_base_reduce_scatter_intra_ring(REDUCESCATTER_ARGS);
+int ompi_coll_base_reduce_scatter_intra_butterfly(REDUCESCATTER_ARGS);
+
+/* Reduce_scatter_block */
+int ompi_coll_base_reduce_scatter_block_basic_linear(REDUCESCATTERBLOCK_ARGS);
+int ompi_coll_base_reduce_scatter_block_intra_recursivedoubling(REDUCESCATTERBLOCK_ARGS);
+int ompi_coll_base_reduce_scatter_block_intra_recursivehalving(REDUCESCATTERBLOCK_ARGS);
+int ompi_coll_base_reduce_scatter_block_intra_butterfly(REDUCESCATTERBLOCK_ARGS);
 
 /* Scan */
+int ompi_coll_base_scan_intra_recursivedoubling(SCAN_ARGS);
+int ompi_coll_base_scan_intra_linear(SCAN_ARGS);
+int ompi_coll_base_scan_intra_recursivedoubling(SCAN_ARGS);
 
 /* Scatter */
 int ompi_coll_base_scatter_intra_basic_linear(SCATTER_ARGS);
 int ompi_coll_base_scatter_intra_binomial(SCATTER_ARGS);
+int ompi_coll_base_scatter_intra_linear_nb(SCATTER_ARGS, int max_reqs);
 
 /* ScatterV */
 
@@ -257,6 +303,25 @@ int mca_coll_base_reduce_local(const void *inbuf, void *inoutbuf, int count,
                                struct ompi_datatype_t * dtype, struct ompi_op_t * op,
                                mca_coll_base_module_t *module);
 
+#if OPAL_ENABLE_FT_MPI
+/* Agreement */
+int ompi_coll_base_agree_noft(void *contrib,
+                              int dt_count,
+                              struct ompi_datatype_t *dt,
+                              struct ompi_op_t *op,
+                              struct ompi_group_t **group, bool update_grp,
+                              struct ompi_communicator_t* comm,
+                              mca_coll_base_module_t *module);
+int ompi_coll_base_iagree_noft(void *contrib,
+                               int dt_count,
+                               struct ompi_datatype_t *dt,
+                               struct ompi_op_t *op,
+                               struct ompi_group_t **group, bool update_grp,
+                               struct ompi_communicator_t* comm,
+                               ompi_request_t **request,
+                               mca_coll_base_module_t *module);
+#endif /* OPAL_ENABLE_FT_MPI */
+
 END_C_DECLS
 
 #define COLL_BASE_UPDATE_BINTREE( OMPI_COMM, BASE_MODULE, ROOT )	\
@@ -298,6 +363,22 @@ do {
     }                                                                                        \
 } while (0)
 
+#define COLL_BASE_UPDATE_KMTREE(OMPI_COMM, BASE_MODULE, ROOT, RADIX)	\
+do {                                                                                         \
+    mca_coll_base_comm_t* coll_comm = (BASE_MODULE)->base_data;                           \
+    if (!((coll_comm->cached_kmtree)                                                       \
+           && (coll_comm->cached_kmtree_root == (ROOT))                                     \
+           && (coll_comm->cached_kmtree_radix == (RADIX))))                                   \
+    {                                                                                        \
+        if (coll_comm->cached_kmtree ) { /* destroy previous k-nomial tree if defined */     \
+            ompi_coll_base_topo_destroy_tree(&(coll_comm->cached_kmtree));                  \
+        }                                                                                    \
+        coll_comm->cached_kmtree = ompi_coll_base_topo_build_kmtree((OMPI_COMM), (ROOT), (RADIX)); \
+        coll_comm->cached_kmtree_root = (ROOT);                                              \
+        coll_comm->cached_kmtree_radix = (RADIX);                                              \
+    }                                                                                        \
+} while (0)
+
 #define COLL_BASE_UPDATE_PIPELINE( OMPI_COMM, BASE_MODULE, ROOT )	\
 do {                                                                                             \
     mca_coll_base_comm_t* coll_comm = (BASE_MODULE)->base_data;                               \
@@ -338,7 +419,7 @@ do {                                                                           \
 } while (0)
 
 /**
- * This macro give a generic way to compute the best count of
+ * This macro gives a generic way to compute the best count of
  * the segment (i.e. the number of complete datatypes that
  * can fit in the specified SEGSIZE). Beware, when this macro
  * is called, the SEGCOUNT should be initialized to the count as
@@ -355,7 +436,7 @@ do {                                                                           \
     }                                                                   \
 
 /**
- * This macro gives a generic wait to compute the well distributed block counts
+ * This macro gives a generic way to compute the well distributed block counts
  * when the count and number of blocks are fixed.
  * Macro returns "early-block" count, "late-block" count, and "split-index"
  * which is the block at which we switch from "early-block" count to
@@ -416,6 +497,11 @@ struct mca_coll_base_comm_t {
     ompi_coll_tree_t *cached_in_order_bmtree;
     int cached_in_order_bmtree_root;
 
+    /* k-nomial tree */
+    ompi_coll_tree_t *cached_kmtree;
+    int cached_kmtree_root;
+    int cached_kmtree_radix;
+
     /* chained tree (fanout followed by pipelines) */
     ompi_coll_tree_t *cached_chain;
     int cached_chain_root;
@@ -433,7 +519,7 @@ OMPI_DECLSPEC OBJ_CLASS_DECLARATION(mca_coll_base_comm_t);
 
 /**
  * Free all requests in an array. As these requests are usually used during
- * collective communications, and as on a succesful collective they are
+ * collective communications, and as on a successful collective they are
  * expected to be released during the corresponding wait, the array should
  * generally be empty. However, this function might be used on error conditions
  * where it will allow a correct cleanup.
@@ -446,6 +532,20 @@ static inline void ompi_coll_base_free_reqs(ompi_request_t **reqs, int count)
 
     for (int i = 0; i < count; ++i) {
         if( MPI_REQUEST_NULL != reqs[i] ) {
+#if OPAL_ENABLE_FT_MPI
+            if( MPI_ERR_PROC_FAILED == reqs[i]->req_status.MPI_ERROR
+             || MPI_ERR_PROC_FAILED_PENDING == reqs[i]->req_status.MPI_ERROR
+             || MPI_ERR_REVOKED == reqs[i]->req_status.MPI_ERROR ) {
+                /* We cannot just 'free' and forget, as the PML/BTLS would still
+                 * be updating the request buffer after we return from the MPI
+                 * call!
+                 * For other errors that do not have a well defined post-error
+                 * behavior, calling the cancel/wait could deadlock, so we just
+                 * free, as this is the best that can be done in this case. */
+                ompi_request_cancel(reqs[i]);
+                ompi_request_wait(&reqs[i], MPI_STATUS_IGNORE);
+            } else /* this 'else' intentionally spills outside the ifdef */
+#endif /* OPAL_ENABLE_FT_MPI */
             ompi_request_free(&reqs[i]);
         }
     }
@@ -453,8 +553,8 @@ static inline void ompi_coll_base_free_reqs(ompi_request_t **reqs, int count)
 
 /**
  * Return the array of requests on the data. If the array was not initialized
- * or if it's size was too small, allocate it to fit the requested size.
+ * or if its size was too small, allocate it to fit the requested size.
  */
-ompi_request_t** coll_base_comm_get_reqs(mca_coll_base_comm_t* data, int nreqs);
+ompi_request_t** ompi_coll_base_comm_get_reqs(mca_coll_base_comm_t* data, int nreqs);
 
 #endif /* MCA_COLL_BASE_EXPORT_H */
