@@ -141,13 +141,12 @@ static void mca_coll_ucc_module_destruct(mca_coll_ucc_module_t *ucc_module)
 #define SAVE_PREV_COLL_API(__api) do {                                                       \
         ucc_module->previous_ ## __api            = comm->c_coll->coll_ ## __api;            \
         ucc_module->previous_ ## __api ## _module = comm->c_coll->coll_ ## __api ## _module; \
-        if (!comm->c_coll->coll_ ## __api || !comm->c_coll->coll_ ## __api ## _module) {     \
-            return OMPI_ERROR;                                                               \
+        if (comm->c_coll->coll_ ## __api && comm->c_coll->coll_ ## __api ## _module) {       \
+            OBJ_RETAIN(ucc_module->previous_ ## __api ## _module);                           \
         }                                                                                    \
-        OBJ_RETAIN(ucc_module->previous_ ## __api ## _module);                               \
     } while(0)
 
-static int mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
+static void mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
 {
     ompi_communicator_t *comm = ucc_module->comm;
     SAVE_PREV_COLL_API(allreduce);
@@ -178,7 +177,6 @@ static int mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
     SAVE_PREV_COLL_API(iscatterv);
     SAVE_PREV_COLL_API(scatter);
     SAVE_PREV_COLL_API(iscatter);
-    return OMPI_SUCCESS;
 }
 
 /*
@@ -293,6 +291,9 @@ static int mca_coll_ucc_init_ctx() {
     ucc_thread_mode_t             tm_requested;
     ucc_lib_params_t              lib_params;
     ucc_context_params_t          ctx_params;
+    unsigned                      ucc_api_major, ucc_api_minor, ucc_api_patch;
+
+    ucc_get_version(&ucc_api_major, &ucc_api_minor, &ucc_api_patch);
 
     tm_requested           = ompi_mpi_thread_multiple ? UCC_THREAD_MULTIPLE :
                                                         UCC_THREAD_SINGLE;
@@ -356,6 +357,15 @@ static int mca_coll_ucc_init_ctx() {
         goto cleanup_lib;
     }
 
+    if (ucc_api_major > 1 || (ucc_api_major == 1 && ucc_api_minor >= 6)) {
+        sprintf(str_buf, "%u", opal_process_info.my_local_rank);
+        if (UCC_OK != ucc_context_config_modify(ctx_config, NULL, "NODE_LOCAL_ID",
+                                                str_buf)) {
+            UCC_ERROR("UCC context config modify failed for node_local_id");
+            goto cleanup_lib;
+        }
+    }
+
     if (UCC_OK != ucc_context_create(cm->ucc_lib, &ctx_params,
                                      ctx_config, &cm->ucc_context)) {
         UCC_ERROR("UCC context create failed");
@@ -470,11 +480,6 @@ static int mca_coll_ucc_module_enable(mca_coll_base_module_t *module,
                 (void*)comm, (long long unsigned)team_params.id,
                 ompi_comm_size(comm));
 
-    if (OMPI_SUCCESS != mca_coll_ucc_save_coll_handlers(ucc_module)){
-        UCC_ERROR("mca_coll_ucc_save_coll_handlers failed");
-        goto err;
-    }
-
     if (UCC_OK != ucc_team_create_post(&cm->ucc_context, 1,
                                        &team_params, &ucc_module->ucc_team)) {
         UCC_ERROR("ucc_team_create_post failed");
@@ -495,6 +500,9 @@ static int mca_coll_ucc_module_enable(mca_coll_base_module_t *module,
         UCC_ERROR("ucc ompi_attr_set_c failed");
         goto err;
     }
+
+    mca_coll_ucc_save_coll_handlers(ucc_module);
+
     return OMPI_SUCCESS;
 
 err:
