@@ -2,17 +2,19 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "ompi_config.h"
@@ -25,18 +27,17 @@
 #include "ompi/attribute/attribute.h"
 #include "ompi/win/win.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Win_set_attr = PMPI_Win_set_attr
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Win_set_attr PMPI_Win_set_attr
 #endif
 
 static const char FUNC_NAME[] = "MPI_Win_set_attr";
 
 
-int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val) 
+int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val)
 {
     int ret;
 
@@ -44,13 +45,11 @@ int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val)
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if (ompi_win_invalid(win)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_WIN, FUNC_NAME);
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_WIN, FUNC_NAME);
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
-    ret = ompi_attr_set_c(WIN_ATTR, win, &win->w_keyhash, 
-                          win_keyval, attribute_val, false, true);
-    OMPI_ERRHANDLER_RETURN(ret, win, MPI_ERR_OTHER, FUNC_NAME);  
+    ret = ompi_attr_set_c(WIN_ATTR, win, &win->w_keyhash,
+                          win_keyval, attribute_val, false);
+    OMPI_ERRHANDLER_RETURN(ret, win, MPI_ERR_OTHER, FUNC_NAME);
 }
