@@ -10,12 +10,14 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2015-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2015-2016 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Intel, Inc. All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2019       FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -86,7 +88,6 @@
 ompi_java_globals_t ompi_java = {0};
 int ompi_mpi_java_eager = 65536;
 opal_free_list_t ompi_java_buffers = {{{0}}};
-static void *libmpi = NULL;
 
 static void bufferConstructor(ompi_java_buffer_t *item)
 {
@@ -107,27 +108,6 @@ OBJ_CLASS_INSTANCE(ompi_java_buffer_t,
  * Class:    mpi_MPI
  * Method:   loadGlobalLibraries
  *
- * Java implementations typically default to loading dynamic
- * libraries strictly to a local namespace. This breaks the
- * Open MPI model where components reference back up to the
- * base libraries (e.g., libmpi) as it requires that the
- * symbols in those base libraries be globally available.
- *
- * One option, of course, is to build with --disable-dlopen.
- * However, this would preclude the ability to pickup 3rd-party
- * binary plug-ins at time of execution. This is a valuable
- * capability that would be a negative factor towards use of
- * the Java bindings.
- *
- * The other option is to explicitly dlopen libmpi ourselves
- * and instruct dlopen to add all those symbols to the global
- * namespace. This must be done prior to calling any MPI
- * function (e.g., MPI_Init) or else Java will have already
- * loaded the library to the local namespace. So create a
- * special JNI entry point that just loads the required libmpi
- * to the global namespace and call it first (see MPI.java),
- * thus making all symbols available to subsequent dlopen calls
- * when opening OMPI components.
  */
 jint JNI_OnLoad(JavaVM *vm, void *reserved)
 {
@@ -135,43 +115,9 @@ jint JNI_OnLoad(JavaVM *vm, void *reserved)
     // the library (see comment in the function for more detail).
     opal_init_psm();
 
-    libmpi = dlopen("libmpi." OPAL_DYN_LIB_SUFFIX, RTLD_NOW | RTLD_GLOBAL);
-
-#if defined(HAVE_DL_INFO) && defined(HAVE_LIBGEN_H)
-    /*
-     * OS X El Capitan does not propagate DYLD_LIBRARY_PATH to children any more
-     * so if previous dlopen failed, try to open libmpi in the same directory
-     * than the current libmpi_java
-     */
-    if(NULL == libmpi) {
-        Dl_info info;
-        if(0 != dladdr((void *)JNI_OnLoad, &info)) {
-            char libmpipath[OPAL_PATH_MAX];
-            char *libmpijavapath = strdup(info.dli_fname);
-            if (NULL != libmpijavapath) {
-                snprintf(libmpipath, OPAL_PATH_MAX-1, "%s/libmpi." OPAL_DYN_LIB_SUFFIX, dirname(libmpijavapath));
-                free(libmpijavapath);
-                libmpi = dlopen(libmpipath, RTLD_NOW | RTLD_GLOBAL);
-            }
-        }
-    }
-#endif
-
-    if(NULL == libmpi)
-    {
-        fprintf(stderr, "Java bindings failed to load libmpi: %s\n",dlerror());
-        exit(1);
-    }
-
     return JNI_VERSION_1_6;
 }
 
-void JNI_OnUnload(JavaVM *vm, void *reserved)
-{
-    if(libmpi != NULL)
-        dlclose(libmpi);
-}
-
 static void initFreeList(void)
 {
     OBJ_CONSTRUCT(&ompi_java_buffers, opal_free_list_t);
@@ -316,7 +262,14 @@ JNIEXPORT jobjectArray JNICALL Java_mpi_MPI_Init_1jni(
     }
 
     int rc = MPI_Init(&len, &sargs);
-    ompi_java_exceptionCheck(env, rc);
+    
+    if(ompi_java_exceptionCheck(env, rc)) {
+        for(i = 0; i < len; i++)
+            free (sargs[i]);
+        free(sargs);
+        return NULL;
+    }
+
     mca_base_var_register("ompi", "mpi", "java", "eager",
                           "Java buffers eager size",
                           MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
@@ -360,7 +313,13 @@ JNIEXPORT jint JNICALL Java_mpi_MPI_InitThread_1jni(
 
     int provided;
     int rc = MPI_Init_thread(&len, &sargs, required, &provided);
-    ompi_java_exceptionCheck(env, rc);
+    
+    if(ompi_java_exceptionCheck(env, rc)) {
+        for(i = 0; i < len; i++)
+            free (sargs[i]);
+        free(sargs);
+        return -1;
+    }
 
     findClasses(env);
     initFreeList();
@@ -714,6 +673,39 @@ static void* getReadPtrvRank(
     return ptr;
 }
 
+static void* getReadPtrwRank(
+        ompi_java_buffer_t **item, JNIEnv *env, jobject buf,
+        int *offsets, int *counts, int *displs, int size,
+        int rank, MPI_Datatype *types, int *baseTypes)
+{
+    int  extent  = getTypeExtent(env, types[rank]),
+         length  = getCountv(counts, displs, size);
+    void *ptr    = getBuffer(env, item, length);
+    int  rootOff = offsets[rank] + displs[rank];
+
+    if(opal_datatype_is_contiguous_memory_layout(&types[rank]->super, counts[rank]))
+    {
+        int  rootLength = extent * counts[rank];
+        void *rootPtr   = (char*)ptr + displs[rank];
+        getArrayRegion(env, buf, baseTypes[rank], rootOff, rootLength, rootPtr);
+    }
+    else
+    {
+        void *inBuf, *inBase;
+        inBuf = ompi_java_getArrayCritical(&inBase, env, buf, rootOff);
+
+        int rc = opal_datatype_copy_content_same_ddt(
+                 &types[rank]->super, counts[rank], ptr, inBuf);
+
+        ompi_java_exceptionCheck(env,
+                rc==OPAL_SUCCESS ? OMPI_SUCCESS : OMPI_ERROR);
+
+        (*env)->ReleasePrimitiveArrayCritical(env, buf, inBase, JNI_ABORT);
+    }
+
+    return ptr;
+}
+
 static void* getReadPtrvAll(
         ompi_java_buffer_t **item, JNIEnv *env, jobject buf,
         int offset, int *counts, int *displs, int size,
@@ -758,6 +750,49 @@ static void* getReadPtrvAll(
     return ptr;
 }
 
+static void* getReadPtrwAll(
+        ompi_java_buffer_t **item, JNIEnv *env, jobject buf,
+        int *offsets, int *counts, int *displs, int size,
+        MPI_Datatype *types, int *baseTypes)
+{
+
+    int length  = getCountv(counts, displs, size);
+    void *ptr    = getBuffer(env, item, length);
+
+    for(int i = 0; i < size; i++)
+    {
+        int extent  = getTypeExtent(env, types[i]);
+
+        if(opal_datatype_is_contiguous_memory_layout(&types[i]->super, 2))
+        {
+            int   iOff = offsets[i] + displs[i],
+                  iLen = extent * counts[i];
+            void *iPtr = (char*)ptr + displs[i];
+            getArrayRegion(env, buf, baseTypes[i], iOff, iLen, iPtr);
+        }
+        else
+        {
+            void *bufPtr, *bufBase;
+            bufPtr = ompi_java_getArrayCritical(&bufBase, env, buf, offsets[i]);
+
+            int   iOff = displs[i];
+            char *iBuf = iOff + (char*)bufPtr,
+                 *iPtr = iOff + (char*)ptr;
+
+            int rc = opal_datatype_copy_content_same_ddt(
+                     &types[i]->super, counts[i], iPtr, iBuf);
+
+            ompi_java_exceptionCheck(env,
+                    rc==OPAL_SUCCESS ? OMPI_SUCCESS : OMPI_ERROR);
+
+            (*env)->ReleasePrimitiveArrayCritical(env, buf, bufBase, JNI_ABORT);
+        }
+
+    }
+
+    return ptr;
+}
+
 static void* getWritePtr(ompi_java_buffer_t **item, JNIEnv *env,
                          int count, MPI_Datatype type)
 {
@@ -777,6 +812,14 @@ static void* getWritePtrv(ompi_java_buffer_t **item, JNIEnv *env,
     return getBuffer(env, item, length);
 }
 
+static void* getWritePtrw(ompi_java_buffer_t **item, JNIEnv *env,
+                          int *counts, int *displs, int size, MPI_Datatype *types)
+{
+    int length = getCountv(counts, displs, size);
+
+    return getBuffer(env, item, length);
+}
+
 void ompi_java_getReadPtr(
         void **ptr, ompi_java_buffer_t **item, JNIEnv *env, jobject buf,
         jboolean db, int offset, int count, MPI_Datatype type, int baseType)
@@ -852,6 +895,39 @@ void ompi_java_getReadPtrv(
     }
 }
 
+void ompi_java_getReadPtrw(
+        void **ptr, ompi_java_buffer_t **item, JNIEnv *env,
+        jobject buf, jboolean db, int *offsets, int *counts, int *displs,
+        int size, int rank, MPI_Datatype *types, int *baseTypes)
+{
+    int i;
+
+    if(buf == NULL)
+    {
+        /* Allow NULL buffers to send/recv 0 items as control messages. */
+        *ptr  = NULL;
+        *item = NULL;
+    }
+    else if(db)
+    {
+        for(i = 0; i < size; i++){
+            assert(offsets[i] == 0);
+        }
+        *ptr  = (*env)->GetDirectBufferAddress(env, buf);
+        *item = NULL;
+    }
+    else if(rank == -1)
+    {
+        *ptr = getReadPtrwAll(item, env, buf, offsets, counts,
+                              displs, size, types, baseTypes);
+    }
+    else
+    {
+        *ptr = getReadPtrwRank(item, env, buf, offsets, counts,
+                               displs, size, rank, types, baseTypes);
+    }
+}
+
 void ompi_java_releaseReadPtr(
         void *ptr, ompi_java_buffer_t *item, jobject buf, jboolean db)
 {
@@ -901,6 +977,27 @@ void ompi_java_getWritePtrv(
     }
 }
 
+void ompi_java_getWritePtrw(
+        void **ptr, ompi_java_buffer_t **item, JNIEnv *env, jobject buf,
+        jboolean db, int *counts, int *displs, int size, MPI_Datatype *types)
+{
+    if(buf == NULL)
+    {
+        /* Allow NULL buffers to send/recv 0 items as control messages. */
+        *ptr  = NULL;
+        *item = NULL;
+    }
+    else if(db)
+    {
+        *ptr  = (*env)->GetDirectBufferAddress(env, buf);
+        *item = NULL;
+    }
+    else
+    {
+        *ptr = getWritePtrw(item, env, counts, displs, size, types);
+    }
+}
+
 void ompi_java_releaseWritePtr(
         void *ptr, ompi_java_buffer_t *item, JNIEnv *env, jobject buf,
         jboolean db, int offset, int count, MPI_Datatype type, int baseType)
@@ -975,6 +1072,49 @@ void ompi_java_releaseWritePtrv(
     releaseBuffer(ptr, item);
 }
 
+void ompi_java_releaseWritePtrw(
+        void *ptr, ompi_java_buffer_t *item, JNIEnv *env,
+        jobject buf, jboolean db, int *offsets, int *counts, int *displs,
+        int size, MPI_Datatype *types, int *baseTypes)
+{
+    if(db || !buf || !ptr)
+        return;
+
+    int i;
+
+    for(i = 0; i < size; i++)
+    {
+        int extent = getTypeExtent(env, types[i]);
+
+        if(opal_datatype_is_contiguous_memory_layout(&types[i]->super, 2))
+        {
+            int   iOff = offsets[i] + displs[i],
+                  iLen = extent * counts[i];
+            void *iPtr = (char*)ptr + displs[i];
+            setArrayRegion(env, buf, baseTypes[i], iOff, iLen, iPtr);
+        }
+        else
+        {
+            void *bufPtr, *bufBase;
+
+            bufPtr = ompi_java_getArrayCritical(&bufBase, env, buf, offsets[i]);
+            int   iOff = displs[i];
+            char *iBuf = iOff + (char*)bufPtr,
+                 *iPtr = iOff + (char*)ptr;
+
+            int rc = opal_datatype_copy_content_same_ddt(
+                     &types[i]->super, counts[i], iBuf, iPtr);
+
+            ompi_java_exceptionCheck(env,
+                    rc==OPAL_SUCCESS ? OMPI_SUCCESS : OMPI_ERROR);
+
+            (*env)->ReleasePrimitiveArrayCritical(env, buf, bufBase, 0);
+        }
+
+    }
+    releaseBuffer(ptr, item);
+}
+
 jobject ompi_java_Integer_valueOf(JNIEnv *env, jint i)
 {
     return (*env)->CallStaticObjectMethod(env,
@@ -1126,6 +1266,18 @@ void ompi_java_releasePtrArray(JNIEnv *env, jlongArray array,
     (*env)->ReleaseLongArrayElements(env, array, jptr, 0);
 }
 
+/* This method checks whether an MPI or JNI exception has occurred.
+ * If an exception occurs, the C code will continue running.  Once
+ * code execution returns to Java code, an exception is immediately
+ * thrown.  Since an exception has occurred somewhere in the C code,
+ * the object that is returned from C may not be valid.  This is not
+ * an issue, however, as the assignment operation will not be
+ * executed.  The results of this method need not be checked if the
+ * only following code cleans up memory and then returns to Java.
+ * If existing objects are changed after a call to this method, the
+ * results need to be checked and, if an error has occurred, the
+ * code should instead cleanup any memory and return.
+ */
 jboolean ompi_java_exceptionCheck(JNIEnv *env, int rc)
 {
     jboolean jni_exception;
