@@ -2,6 +2,10 @@
 /*
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2021      Amazon.com, Inc. or its affiliates.  All Rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -9,18 +13,17 @@
  * $HEADER$
  */
 
+#include "ompi_config.h"
+
 #include "ompi/mpi/tool/mpit-internal.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_T_pvar_stop = PMPI_T_pvar_stop
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/tool/profile/defines.h"
+#define MPI_T_pvar_stop PMPI_T_pvar_stop
 #endif
 
-static const char FUNC_NAME[] = "MPI_T_pvar_stop";
-
 static int pvar_handle_stop (mca_base_pvar_handle_t *handle)
 {
     if (OPAL_SUCCESS != mca_base_pvar_handle_stop (handle)) {
@@ -32,13 +35,13 @@ static int pvar_handle_stop (mca_base_pvar_handle_t *handle)
 
 int MPI_T_pvar_stop(MPI_T_pvar_session session, MPI_T_pvar_handle handle)
 {
-    int ret;
+    int ret = MPI_SUCCESS;
 
     if (!mpit_is_initialized ()) {
         return MPI_T_ERR_NOT_INITIALIZED;
     }
 
-    mpit_lock ();
+    ompi_mpit_lock ();
 
     if (MPI_T_PVAR_ALL_HANDLES == handle) {
         OPAL_LIST_FOREACH(handle, &session->handles, mca_base_pvar_handle_t) {
@@ -55,7 +58,7 @@ int MPI_T_pvar_stop(MPI_T_pvar_session session, MPI_T_pvar_handle handle)
         ret = pvar_handle_stop (handle);
     }
 
-    mpit_unlock ();
+    ompi_mpit_unlock ();
 
     return ompit_opal_to_mpit_error (ret);
 }
