@@ -29,9 +29,9 @@
 
 #include <stdio.h>
 
-#include "opal/sys/atomic.h"
 #include "opal/class/opal_object.h"
 #include "opal/constants.h"
+#include "opal/sys/atomic.h"
 
 /*
  * Instantiation of class descriptor for the base class.  This is
@@ -55,28 +55,26 @@ int opal_class_init_epoch = 1;
 /*
  * Local variables
  */
-static opal_atomic_lock_t class_lock = { { OPAL_ATOMIC_LOCK_UNLOCKED } };
-static void** classes = NULL;
+static opal_atomic_lock_t class_lock = OPAL_ATOMIC_LOCK_INIT;
+static void **classes = NULL;
 static int num_classes = 0;
 static int max_classes = 0;
 static const int increment = 10;
 
-
 /*
  * Local functions
  */
 static void save_class(opal_class_t *cls);
 static void expand_array(void);
 
-
 /*
  * Lazy initialization of class descriptor.
  */
 void opal_class_initialize(opal_class_t *cls)
 {
     opal_class_t *c;
-    opal_construct_t* cls_construct_array;
-    opal_destruct_t* cls_destruct_array;
+    opal_construct_t *cls_construct_array;
+    opal_destruct_t *cls_destruct_array;
     int cls_construct_array_count;
     int cls_destruct_array_count;
     int i;
@@ -107,12 +105,12 @@ void opal_class_initialize(opal_class_t *cls)
 
     cls->cls_depth = 0;
     cls_construct_array_count = 0;
-    cls_destruct_array_count  = 0;
+    cls_destruct_array_count = 0;
     for (c = cls; c; c = c->cls_parent) {
-        if( NULL != c->cls_construct ) {
+        if (NULL != c->cls_construct) {
             cls_construct_array_count++;
         }
-        if( NULL != c->cls_destruct ) {
+        if (NULL != c->cls_destruct) {
             cls_destruct_array_count++;
         }
         cls->cls_depth++;
@@ -123,38 +121,35 @@ void opal_class_initialize(opal_class_t *cls)
      * plus for each a NULL-sentinel
      */
 
-    cls->cls_construct_array =
-        (void (**)(opal_object_t*))malloc((cls_construct_array_count +
-                                           cls_destruct_array_count + 2) *
-                                          sizeof(opal_construct_t) );
+    cls->cls_construct_array = (void (**)(opal_object_t *)) malloc(
+        (cls_construct_array_count + cls_destruct_array_count + 2) * sizeof(opal_construct_t));
     if (NULL == cls->cls_construct_array) {
         perror("Out of memory");
         exit(-1);
     }
-    cls->cls_destruct_array =
-        cls->cls_construct_array + cls_construct_array_count + 1;
+    cls->cls_destruct_array = cls->cls_construct_array + cls_construct_array_count + 1;
 
     /*
      * The constructor array is reversed, so start at the end
      */
 
     cls_construct_array = cls->cls_construct_array + cls_construct_array_count;
-    cls_destruct_array  = cls->cls_destruct_array;
+    cls_destruct_array = cls->cls_destruct_array;
 
     c = cls;
-    *cls_construct_array = NULL;  /* end marker for the constructors */
+    *cls_construct_array = NULL; /* end marker for the constructors */
     for (i = 0; i < cls->cls_depth; i++) {
-        if( NULL != c->cls_construct ) {
+        if (NULL != c->cls_construct) {
             --cls_construct_array;
             *cls_construct_array = c->cls_construct;
         }
-        if( NULL != c->cls_destruct ) {
+        if (NULL != c->cls_destruct) {
             *cls_destruct_array = c->cls_destruct;
             cls_destruct_array++;
         }
         c = c->cls_parent;
     }
-    *cls_destruct_array = NULL;  /* end marker for the destructors */
+    *cls_destruct_array = NULL; /* end marker for the destructors */
 
     cls->cls_initialized = opal_class_init_epoch;
     save_class(cls);
@@ -164,7 +159,6 @@ void opal_class_initialize(opal_class_t *cls)
     opal_atomic_unlock(&class_lock);
 }
 
-
 /*
  * Note that this is finalize for *all* classes.
  */
@@ -193,7 +187,6 @@ int opal_class_finalize(void)
     return OPAL_SUCCESS;
 }
 
-
 static void save_class(opal_class_t *cls)
 {
     if (num_classes >= max_classes) {
@@ -204,13 +197,12 @@ static void save_class(opal_class_t *cls)
     ++num_classes;
 }
 
-
 static void expand_array(void)
 {
     int i;
 
     max_classes += increment;
-    classes = (void**)realloc(classes, sizeof(opal_class_t*) * max_classes);
+    classes = (void **) realloc(classes, sizeof(opal_class_t *) * max_classes);
     if (NULL == classes) {
         perror("class malloc failed");
         exit(-1);
@@ -219,4 +211,3 @@ static void expand_array(void)
         classes[i] = NULL;
     }
 }
-
